/*-----------------------------------------------------------------------------*
// include
*-----------------------------------------------------------------------------*/
#include "harktool.h"




/*-----------------------------------------------------------------------------*
// extern
*-----------------------------------------------------------------------------*/
extern struct GLOBAL  g;






/*-----------------------------------------------------------------------------*
* インパルス応答から伝達関数生成
*
*
*
*-----------------------------------------------------------------------------*/
void harktoolcli_calctfimp()
{


	harktool_LogI( "harktoolcli_calctfimp start" );




    if(      strlen( g.argv.mic_file ) == 0 )
    {
        harktool_LogE("argument error [--mic-list]");
        harktool_help( "calctfimp" );
        harktool_Exit(EXIT_FAILURE);
    }
    else if( strlen( g.argv.impulse_file ) == 0 ) 
    {
        harktool_LogE("argument error [--impulse-list]");
        harktool_help( "calctfimp" );
        harktool_Exit(EXIT_FAILURE);

    }
    else if( strlen( g.argv.output ) == 0 )
    {
        harktool_LogE("argument error [--output-file]");
        harktool_help( "calctfimp" );
        harktool_Exit(EXIT_FAILURE);

    }
    else if( g.argv.tf_type != LOC && g.argv.tf_type != SEP  && g.argv.tf_type != LOC_SEP )
    {
        harktool_LogE("argument error [--output-type]");
        harktool_help( "calctfimp" );
        harktool_Exit(EXIT_FAILURE);
    }



    /*--------------------------------------------------*/
    //伝達関数作成
    /*--------------------------------------------------*/
    harktoolcli_calctfimpExec();


    /*--------------------------------------------------*/
    /* end proc                                         */
    /*--------------------------------------------------*/
    harkio_Log_printf(harkio_Log_Info, "@ harktoolcli_calctfimp normal end");

    harktool_Exit(EXIT_SUCCESS);


}//end







/*-----------------------------------------------------------------------------*
* 伝達関数作成処理
*
*
*
*-----------------------------------------------------------------------------*/
void harktoolcli_calctfimpExec( )
{



    /*--------------------------------------------------*/
    // micファイル(microphones.xml)->XML構造体へ
    /*--------------------------------------------------*/
    g.mic_xml     = harkio_XML_newFromXML( g.argv.mic_file , "" );


    /*--------------------------------------------------*/
    // impulse list ファイル(source.xml)->XML構造体へ
    /*--------------------------------------------------*/
    g.impulse_xml = harkio_XML_newFromXML( g.argv.impulse_file , "" );


    /*--------------------------------------------------*/
    // position type
    /*--------------------------------------------------*/
    g.PosType = IMPULSE;

    /*--------------------------------------------------*/
    //mic数
    /*--------------------------------------------------*/
    g.Nmic = harkio_Positions_getSize( g.mic_xml->poses );


    /*--------------------------------------------------*/
    //音源数
    /*--------------------------------------------------*/
    g.Nsrc = harkio_Positions_getSize( g.impulse_xml->poses );




    /*--------------------------------------------------*/
    // Config取得
    /*--------------------------------------------------*/
    g.Nfft           = harkio_Config_getNfft( g.impulse_xml->cfg );
    g.TspLength      = harkio_Config_getTSPLength( g.impulse_xml->cfg );
    g.SamplingRate   = harkio_Config_getSamplingRate( g.impulse_xml->cfg );
    g.SignalMax      = harkio_Config_getSignalMax( g.impulse_xml->cfg );
    g.SyncAverage    = harkio_Config_getSynchronousAverage( g.impulse_xml->cfg );
    g.OrgTspPath     = harkio_Config_getTSPpath( g.impulse_xml->cfg );
    g.TspOffset      = harkio_Config_getTSPoffset( g.impulse_xml->cfg );
    g.PeakSearchFrom = harkio_Config_getPeakSearchFrom( g.impulse_xml->cfg );
    g.PeakSearchTo   = harkio_Config_getPeakSearchTo( g.impulse_xml->cfg );


    g.IRsize = g.Nsrc * g.Nmic * g.Nfft ;
    g.TFsize = g.Nsrc * g.Nmic * ( g.Nfft/2 + 1) ;




    /*--------------------------------------------------*/
    /* 領域確保                                         */
    /*--------------------------------------------------*/

    // impulse応答用領域
    // npos(72) x Nmic(8) x Nfft(512) x sizeof(float) 分の領域確保
    harktool_new( IR_VECT );

    // impulse応答 計算用領域
    // npos(72) x Nmic(8) x Nfft(512) x sizeof(float) 分の作業領域確保
    harktool_new( TMP_IR_VECT );

    // impulse応答 BACKUP領域
    harktool_new( BACK_IR_VECT );

    // 伝達関数用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の領域確保
    harktool_new( TF_VECT );

    // 伝達関数 作業用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の作業領域確保
    harktool_new( TMP_TF_VECT );

    //伝達関数作成用TEMP領域
    harktool_new( BUFFER );

    //Mic position
    harktool_new( MIC_POS_VECT );

    //SRC position
    harktool_new( SRC_POS_VECT );

    //SORT
    harktool_new( SORT_POS_ID );


    // インパルス応答ファイルのPATH格納領域(tmp.flt_path)
    harktool_new( FLT_PATH );



    /*--------------------------------------------------*/
    //  FFT data set
    /*--------------------------------------------------*/
    setFftData();


    /*--------------------------------------------------*/
    /*  Create Directory                                */
    /*--------------------------------------------------*/
    harktool_createTmpPath() ;


    /*--------------------------------------------------*/
    // id を昇順に並べ替えてpositionをセット
    /*--------------------------------------------------*/
    //MIC ID 並べ替え
    harktool_setMicPosId();

    //mic positionセット + 極座標->直交座標変換処理
    harktool_setMicPos( );

    //pos->idを昇順に並べ替え(IrPosId)
    harktool_setIrPosId();


    /*--------------------------------------------------*/
	/* wavファイルからIR変換                               */
    /*--------------------------------------------------*/
	harktool_setIRvectFromWav();


    /*--------------------------------------------------*/
	//IRvect back up
    /*--------------------------------------------------*/
	harktool_copyIRvectToBackIRvect();
    


    switch(g.argv.tf_type) //tf_type:SEP,LOC,LOC_SEP
    {
    case LOC:
		harktool_genMUSIC();
		break;
		

    case SEP:
		harktool_genGSS();
		break;

    case LOC_SEP:

		//Loc
		harktool_genMUSIC();
		
		//data free
		harktool_delete(IR_VECT);
		harktool_delete(TMP_IR_VECT);
		harktool_delete(TF_VECT);
		harktool_delete(TMP_TF_VECT);
		harktool_delete(BUFFER);
		
		harktool_new( IR_VECT );     // impulse応答用領域
		harktool_new( TMP_IR_VECT ); // impulse応答 計算用領域
		harktool_new( TF_VECT );     // 伝達関数用領域
		harktool_new( TMP_TF_VECT ); // 伝達関数 作業用領域
		harktool_new( BUFFER );      // 伝達関数作成用TEMP領域


		//recovery
		harktool_copyBackIRvectToIRvect();
		
		//sep
		harktool_genGSS();
		break;

	}




    /*--------------------------------------------------*/
    // 伝達関数をZipファイルに出力
    /*--------------------------------------------------*/
    harktool_writeTF();


    /*--------------------------------------------------*/
    /*  正規化を書き込む                                */
    /*--------------------------------------------------*/
    createReadMeToZipFile(g.argv.output);


}//main end






