#include<stdio.h>
#include<stdlib.h>
#include<malloc.h>
#include<sndfile.h>



//---------------------------------------//
// open wave file
//---------------------------------------//
SNDFILE *waveopen(char *path, int sfmode, SF_INFO *sfinfo)
{

  SNDFILE *fp;

  sfinfo->format = 0;

  if( (fp = sf_open(path, sfmode, sfinfo)) == NULL )
  {
    perror("sf_open");
    return NULL;
  }

  return fp;
}






//---------------------------------------//
// get sndinfo 
//---------------------------------------//
int getwavinfo(char   *fname, 
			   int    *length,
			   int    *ch,
			   int    *samplerate)
{

	SNDFILE *fp;
	SF_INFO sfinfo;
	
	sfinfo.format = 0;
	fp = sf_open( fname, SFM_READ, &sfinfo);

	sf_close(fp);

	*length     = sfinfo.frames;
	*ch         = sfinfo.channels;
	*samplerate = sfinfo.samplerate;

	return 1;
}










//---------------------------------------//
// read wave data
//---------------------------------------//
int readwave(SNDFILE *fp, SF_INFO *sfinfo, int count, double ***wave)
{

  short *data;
  int i,j;
  unsigned long norm;


  switch((sfinfo->format) & (SF_FORMAT_SUBMASK))
  {
  case SF_FORMAT_PCM_16: /* Signed 16 bit data */
    norm = 32768;
    break;

  case SF_FORMAT_PCM_24: /* Signed 24 bit data */
      norm = 8388608; //2^24(8388608)で割る0～1の範囲に正規化
      break;

  case SF_FORMAT_FLOAT :
      norm = 1;
      break;


  default:
      perror("wav format error\n");
      exit(0);
  }

  printf("norm=%ld\n",norm);



    if( (data = (short *)malloc(sizeof(short) * count * sfinfo->channels) )
        == NULL ){
      perror("malloc");
      exit(0);
    }

    if( (*wave = (double **)malloc(sizeof(double *) * sfinfo->channels) )
        == NULL ){
        perror("malloc");
        exit(0);
    }

    for( i =0; i < sfinfo->channels; i++ ){
        if( ((*wave)[i] = (double *)malloc(sizeof(double) * count )) == NULL ){
            perror("malloc");
            exit(0);
        }
    }

    sf_readf_short(fp, data, count);

    for( i = 0; i < count; i++ )
        for( j = 0; j < sfinfo->channels; j++ ){
            (*wave)[j][i] = (double)data[j+i*sfinfo->channels]/norm;
             printf( "readwave: ch=%d wav[%d][%d]=%lf\n",sfinfo->channels, j,i, (*wave)[j][i]);

        }

    free(data);

    return 1;
  }





//---------------------------------------//
// read wave file
//---------------------------------------//
int readwavfile(char   *fname, 
                double ***wave,
				int    *length,
				int    *ch,
                int    *samplerate,
                int    *format)
{

    SNDFILE *fp;
	SF_INFO sfinfo;
	
	sfinfo.format = 0;
	fp = sf_open( fname, SFM_READ, &sfinfo);

    printf("sfinfo.frames=%d\n"    , (int)sfinfo.frames) ;
    printf("sfinfo.channels=%d\n"  ,sfinfo.channels);
    printf("sfinfo.samplerate=%d\n",sfinfo.samplerate);
    printf("sfinfo.format=%d\n"    ,sfinfo.format);



    readwave(fp, &sfinfo, sfinfo.frames, wave);
	


	sf_close(fp);

	*length     = sfinfo.frames;
	*ch         = sfinfo.channels;
	*samplerate = sfinfo.samplerate;
    *format     = sfinfo.format;

    return 1;
}






//---------------------------------------//
// read wave file
//---------------------------------------//
int readwavimpfile(char   *fname,
                double ***wave,
                int    *length,
                int    *ch,
                int    *samplerate,
                int    *format)
{

    SNDFILE *fp;
    SF_INFO sfinfo;

    sfinfo.samplerate = 16000;
    sfinfo.channels = 8;
    sfinfo.format = SF_FORMAT_WAV | SF_FORMAT_FLOAT | SF_ENDIAN_LITTLE;
    sfinfo.sections = -1;
    sfinfo.seekable = -1;


    fp = sf_open( fname, SFM_READ, &sfinfo);

//    printf("impulse sfinfo.frames=%d\n",sfinfo.frames) ;
//    printf("impulse sfinfo.channels=%d\n",sfinfo.channels);
//    printf("impulse sfinfo.samplerate=%d\n",sfinfo.samplerate);
//    printf("impulse sfinfo.format=%d\n",sfinfo.format);


    float *data;
    int i,j;
    int count;

    count = sfinfo.frames;

    //tmp file
    if( (data = (float *)malloc(sizeof(float) * count * sfinfo.channels) ) == NULL )
    {
      perror("malloc");
      exit(0);
    }

    //
    if( (*wave = (double **)malloc(sizeof(double *) * sfinfo.channels) ) == NULL )
    {
        perror("malloc");
        exit(0);
    }


    for( i =0; i < sfinfo.channels; i++ )
    {
        if( ((*wave)[i] = (double *)malloc(sizeof(double) * count )) == NULL )
        {
            perror("malloc");
            exit(0);
        }
    }

    //file read
    sf_readf_float(fp, data, count);

    printf("data[0]=%f\n",data[0]);
    printf("data[1]=%f\n",data[1]);
    printf("data[2]=%f\n",data[2]);




    for( i = 0; i < count; i++ )
    {
        for( j = 0; j < sfinfo.channels; j++ )
        {
            (*wave)[j][i] = ((double)data[j+i*sfinfo.channels]) ;
            printf( "readwave: ch=%d wav[%d][%d]=%lf\n",sfinfo.channels, j,i, (*wave)[j][i]);

        }
    }



    free(data);




    for( j = 0; j < 8; j++ )
    {
        for( i = 0; i < 4096; i++ )
        {
            printf( "readwavimpfile : wav[%d][%d]=%lf\n", j,i, (*wave)[j][i]);

        }
    }



    sf_close(fp);

    *length     = sfinfo.frames;
    *ch         = sfinfo.channels;
    *samplerate = sfinfo.samplerate;
    *format     = sfinfo.format;

    return 1;
}




























//---------------------------------------//
// write wave data
//---------------------------------------//
int writewave(SNDFILE *fp, SF_INFO *sfinfo, int count, double **wave)
{

  short *data;
  int i,j;

  if( (data = (short *)malloc(sizeof(short) * count * sfinfo->channels) ) == NULL )
  {
    perror("malloc");
    exit(0);
  }

  for( i = 0; i < count; i++ )
  {
	  for( j = 0; j < sfinfo->channels; j++ )
	  {
		  data[j+i*sfinfo->channels] = (short)wave[j][i];
	  }
  }
  
  sf_writef_short(fp, data, count);

  free(data);

  return 1;
}






//---------------------------------------//
// write wave file
//---------------------------------------//
int writewavfile(char *fname, double **wave,
				 int length,
				 int ch,
				 int samplerate)
{

	SNDFILE *fp;
	SF_INFO sfinfo;
	
	sfinfo.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
	sfinfo.samplerate = samplerate;
	sfinfo.channels = ch;
	sfinfo.sections = -1;
	sfinfo.seekable = -1;
//	sfinfo.frames = length;
	
	fp = sf_open( fname, SFM_WRITE, &sfinfo);

    writewave(fp, &sfinfo, length, wave);

    sf_close(fp);

	return 1;
}












//---------------------------------------//
// write wave data
//---------------------------------------//
int harktool_writewave(SNDFILE *fp     ,
                       SF_INFO *sfinfo ,
                       int     count   ,
                       double **wave)
{

  short *data;
  int   i,j;

  if( (data = (short *)malloc(sizeof(short) * count * sfinfo->channels) ) == NULL )
  {
    perror("malloc");
    exit(0);
  }

  for( i = 0; i < count; i++ )
  {
      for( j = 0; j < sfinfo->channels; j++ )
      {
          data[j+i*sfinfo->channels] = (short)wave[j][i];
      }
  }

  sf_writef_short(fp, data, count);

  free(data);

  return 1;
}






//---------------------------------------//
// write wave file
//---------------------------------------//
int harktool_writewavfile(char  *fname  ,
                          float *data  ,
                          int    length   ,
                          int    ch       ,
                          int    samplerate )
{

    SF_INFO sfinfo;
    SNDFILE *fp;


    sfinfo.samplerate = samplerate;
    sfinfo.channels   = ch;
    sfinfo.format     = SF_FORMAT_WAV | SF_FORMAT_FLOAT | SF_ENDIAN_LITTLE;
    sfinfo.sections = -1;
    sfinfo.seekable = -1;


    if(sf_format_check(&sfinfo) != 1 )
    {
        fprintf(stderr, "error in format\n");
        exit(1);
    }


    if( (fp = sf_open( fname, SFM_WRITE, &sfinfo)) == NULL )
    {
      perror("sf_open error");
      return -1;
    }


    sf_writef_float( fp, data, length );

    sf_close(fp);

    return 1;
}







#if 0
int main(int argc, char *argv[]){

  SNDFILE *fp;
  int mode = SFM_READ;
  SF_INFO sfinfo;
  short data[16*10];
  short wave[16][10];
  int i, j;

  sfinfo.format = 0;
  fp = sf_open("../Database/SPEECH/ATR_F1_D0_150cm.wav", mode, &sfinfo);

  printf("frame %d\n", sfinfo.frames );
  printf("fs %d\n", sfinfo.samplerate );
  printf("ch %d\n", sfinfo.channels );
  printf("fmt 0x%x\n", sfinfo.format);
  printf("sections %d\n", sfinfo.sections );
  printf("seekable %d\n", sfinfo.seekable );

  sf_readf_short(fp, data, 10);
  sf_close(fp);

  for( i = 0; i < 10; i++ )
    for( j = 0; j < 16; j++ ){
      wave[j][i] = data[j+i*16];
    }

  for( i = 0; i < 16; i++ ){
    printf("Ch %d = ", i );
    for( j = 0; j < 10; j++ ){
      printf("%3d ", wave[i][j] );
    }
    printf("\n");
  }
}
#endif
