#include <stdio.h>  
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <libharkio3.h>
#include <math.h>
#include <libharkio3.h>


#define ZIP_FILE_MAX  7


void StrSort(char *str[ ], int n);

float calcZValue( int z_unit , float re , float im   );
void WriteCoordinate( struct zip  *handle  ,char *fname[] , int f_max , float fs , int nfft ,char *outfile ,int z_axis );
void getZipData( struct zip  *handle , int *nfft ,int *nfs);

#define DISP_REAL     1
#define DISP_AMP     2
#define DISP_db     3
#define DISP_PHASE  4


#define  LOCDIR  "transferFunction/localization/tf"
#define	 SEPDIR  "transferFunction/separation/tf"

#define COMPLEX_LEN 8



void help(void)
{
	printf("                                                \n");
	printf("                                                \n");
	printf("[NAME:]                                         \n");
	printf("  harktoolcli-zip2coord                         \n");
	printf("                                                \n");
	printf("[Description:]                                  \n");
	printf("  Coordinate from transfer function             \n");
	printf("                                                \n");
	printf("[Usage :]                                       \n");
    printf("  harktoolcli-zip2coord tf_type tf_filename(zip)  \n");
	printf("                                                  \n");
    printf("[options]                                                     \n");
    printf("      -t: Type of Transfer function    (-t LOC)    [string]   \n");
 	printf("            LOC : Localization transfer function              \n");
 	printf("            SEP : Separation   transfer function              \n");
    printf("      -z: z_axis type   ( amp: amplitude   db: decibel  phase:phase  [string]  \n");
    printf("      -i: input file                     [string]    \n");
    printf("      -o: output file                    [string]    \n");
//    printf("      -s: Sampling frequency           (-s 16000)    [integer] \n");
//    printf("      -n: Number of fft                (-n 512)      [integer] \n");
	printf("                                                               \n");

}









int main(int argc, char **argv)
{
    int option;

    int        errorp;
    int        index;
    const char *name;
    char msg[MSG_LEN];
    int  flags = 0;
    struct zip  *handle;
    zip_int64_t  num;

    memset(msg,  '\0' , sizeof(msg));


    char infile[2048];
    char outfile[2048];
	char tfdirname[1024];

	int samplingrate ;
	int fft_len ;
	int z_axis = DISP_AMP;
	int tf_type=0;

	char tmp[10][100];
	memset(tmp,0,sizeof(tmp));



    if(argc < 2)
    {
		harkio_Log_printf(harkio_Log_Error , "argument error!!" );
        help();
		exit(EXIT_FAILURE);
    }


    // parse option
    while((option = getopt(argc, argv, "t:z:o:i:s:n:")) != -1)
    {
        switch(option)
        {
        case 't':
        	if( strcmp( optarg , "LOC" ) == 0 ||  strcmp( optarg , "loc" ) == 0 )
        	{
				memset( tfdirname , 0x00 ,sizeof(tfdirname) );
	        	strcpy( tfdirname , LOCDIR );
	        	tf_type=0;
        	}
        	else if( strcmp( optarg , "SEP" ) == 0 || strcmp( optarg , "sep" ) == 0 )
			{
				memset( tfdirname , 0x00 ,sizeof(tfdirname) );
	        	strcpy( tfdirname , SEPDIR );
	        	tf_type=1;
			}            
			else
			{
		        help();
			    exit(EXIT_FAILURE);
			}

        break;

        case 'z':
        	if( strcmp( optarg , "amp" ) == 0 )
        	{
        		z_axis  = DISP_AMP;
        	}
        	else if( strcmp( optarg , "db" ) == 0 )
			{
        		z_axis  = DISP_db;
			}            
        	else if( strcmp( optarg , "phase" ) == 0 )
			{
        		z_axis  = DISP_PHASE;
			}            
			else
			{
		        help();
			    exit(EXIT_FAILURE);
			}

        break;

        case 'i':
			memset( infile , 0x00 , sizeof(infile));
	        strncpy( infile , optarg, strlen(optarg));
        break;

        case 'o':
			memset( outfile , 0x00 , sizeof(outfile));
	        strncpy( outfile , optarg, strlen(optarg));
	        break;

//        case 's':
//			samplingrate = atoi(optarg);
//	        break;
//
//        case 'n':
//			fft_len = atoi(optarg);
//	        break;


        case '?':
            help();
	        exit(EXIT_FAILURE);
        }
    }



    handle = zip_open( infile , ZIP_CHECKCONS, &errorp);


    if(handle == 0x00 )
    {
        sprintf(msg,"can not open zip file [%s]",argv[1]);
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
    }

    num = zip_get_num_entries( handle ,  flags);

	printf("file count : %d\n", (int)num);

    if( num < ZIP_FILE_MAX )
    {
        sprintf(msg,"file count  is different [%s]",argv[1]);
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
    }




	getZipData( handle , &fft_len , &samplingrate);
	
	printf("nfft=%d samplingrate=%d\n",fft_len,samplingrate);
	




	char *fname[num];
	int cnt=0;
	
    for(index=0 ; index<num ; index++)
    {
		name = zip_get_name( handle , index , flags);

		if(name == NULL) continue;		

		if( strncmp( tfdirname , name , strlen(tfdirname) ) == 0)
		{
	    	fname[cnt] = (char*)name;
			cnt++;
		}
    }


    if( cnt == 0 )
    {
    	switch(tf_type)
    	{
    	case 0:
	        sprintf(msg , "not mat file of localization\n" );
			break;			
		case 1:
	        sprintf(msg , "not mat file of separation\n" );
			break;			
		}
		
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
    }





	/*---------------------------------------*/
	//sort
	/*---------------------------------------*/
	StrSort( fname , cnt);


	/* ソート結果を表示 */
	int i;
	printf("\nソート後: \n");		
	for (i = 0; i < cnt; i++)
	{
		printf("%s  \n", fname[i]);
	}
	printf("\n");



	/*---------------------------------------*/
	// file write
	/*---------------------------------------*/
	WriteCoordinate( handle ,fname , cnt , samplingrate , fft_len ,outfile , z_axis);



	/*---------------------------------------*/
	// exit
	/*---------------------------------------*/
	printf("SUCCESS\n");
	
	zip_close(handle);
	exit(EXIT_SUCCESS);

}











/*------------------------------------------------------*/
//
/* n個の文字列をソート */
//void StrSort(char *str[ ], int n)
/*------------------------------------------------------*/
void StrSort(char *str[], int n)
{
    int i, j;
    char *temp;   /* 作業用ポインタ */
	
    for (i = 0; i < n - 1; i++) {
        for (j = i + 1; j < n; j++) {
            if ((strcmp(str[j], str[i])) < 0) {
                temp = str[i];                /* 作業用ポインタを使い */
                str[i] = str[j];              /* i + 1番目に小さい要素と */
                str[j] = temp;                /* str[i] の要素と交換 */
            }
        }
//        for (k = 0; k < n; k++)            /* ソートの途中経過を表示 */
//            printf("%s  ", str[k]);
//        printf("\n");
    }
}








/*---------------------------------------*/
// Z軸計算
/*---------------------------------------*/
float calcZValue( int z_unit , float re , float im   )
{

    char msg[MSG_LEN];


    switch(z_unit)
    {
        case DISP_REAL: 
            return re;
            break;
        case DISP_AMP:
//        printf("calcZValue=%f %f ,  sqlf=%f\n",re,im , sqrtf(re*re + im*im));
        return sqrtf(re*re + im*im);
            break;
        case DISP_db:
            return log10(sqrtf(re*re + im*im)) * 10;
            break;
        case DISP_PHASE:
            return atan2f(im, re) * 180/M_PI;
            break;
        default:
		    sprintf(msg,"z_unit error [%d]", z_unit );
		    harkio_Log_printf(harkio_Log_Error , msg);
		    exit(EXIT_FAILURE);
        break;
    }
	
	return -1;
}







/*------------------------------------------------------*/
//  座標変換
/*------------------------------------------------------*/
void WriteCoordinate(  struct zip  *handle ,  char *fname[] , int f_max , float fs , int nfft , char *outfile , int z_axis)
{

	struct zip_stat zs;	
	struct zip_file* fp;
	char *buff;
    char  tag[33];
    char  type[33];
    int   dim;
    int   rows;
    int   cols;
	FILE *fpw;	
	float real;
	float imag;
	int   i;	
	int   len;
	char  tmp[10];
	char  chrID[6];
	int   src_id;
	float x_weight   = fs/nfft;
	int   x_max      = nfft/2;
	char  wbuff[256];
	float z_data;
	char  msg[MSG_LEN];
	int   no;
	char  out[256];
	

	//heder write 
	if( (fpw = fopen( outfile  , "w" )) == NULL )
	{
	    sprintf(msg,"fopen error [%s]", outfile );
	    harkio_Log_printf(harkio_Log_Error , msg);
	    exit(EXIT_FAILURE);
	}

	memset(wbuff , 0 , sizeof(wbuff));
	sprintf( wbuff , "%s\n", "id,mic,x,y,z" );
	fwrite( wbuff, strlen(wbuff), 1, fpw );

	fclose( fpw );


	for( no=0 ; no < f_max ; no++ )
	{
		//zip open
		if( ( fp = zip_fopen( handle , fname[no] ,0) ) == NULL)
		{
		    sprintf(msg,"fopen error [%s]", outfile );
		    harkio_Log_printf(harkio_Log_Error , msg);
		    exit(EXIT_FAILURE);
		}
	
		zip_stat_init(&zs);

		zip_stat(handle,  fname[no] ,ZIP_STAT_SIZE ,  &zs);

		buff = (char*)malloc(zs.size );

		//zip header
		zip_fread(fp,tag  ,32);
		zip_fread(fp,type ,32);
		zip_fread(fp,&dim ,4);
		zip_fread(fp,&rows,4);
		zip_fread(fp,&cols,4);



		len = ( zs.size 
					- sizeof(tag)
					- sizeof(type)
					- sizeof(dim)
					- sizeof(rows)
					- sizeof(cols) ) / COMPLEX_LEN ; 


		COMPLEX_TYPE data[len];


//		printf("row=%d  cols=%d zs.size=%d  len=%d\n",rows,cols,(int)zs.size , len);


		//set all data
		for( i=0;i< len   ;i++)
		{
			zip_fread(fp, &data[i] , COMPLEX_LEN  );
		}


		if( (fpw = fopen( outfile  , "a" )) == NULL )
		{
		    sprintf(msg,"fopen error [%s]", outfile );
		    harkio_Log_printf(harkio_Log_Error , msg);
		    exit(EXIT_FAILURE);
		}


		//get src id 	
		memset(out , 0 , sizeof(out));
		strcpy(out , fname[no]);

		memset(tmp , 0 , sizeof(tmp));
		memset(chrID , 0 , sizeof(chrID));

		strncpy( tmp   , &out[strlen(out)-9] , 9 );//数時以降取り出し
		strncpy( chrID , tmp , 5 );//数字だけ取り出し
		src_id = atoi(chrID);



		//x軸がHzの場合: 256個
		//y軸がsrcの場合: 72個(matファイル数)
		//z軸がAMPの場合: 

		int cnt=0;
		float x;
		int mic;
		
		
		
		
		for( mic=0 ; mic < rows ; mic++)
		{	
			for( x=0 ; x<= x_max ; x++)
			{
				float x_data = x * x_weight;
				float y_data = no * 0.2 ;
	
				//z軸
				real = crealf(data[cnt]);
				imag = cimagf( data[cnt]);

				z_data = calcZValue( z_axis , real  , imag );

				cnt++;

				memset(wbuff,0,sizeof(wbuff));
				sprintf( wbuff ,   "%d , %d , %f , %f , %f\n", src_id , mic , x_data , y_data , z_data );
//				printf("%s",wbuff);
				fwrite( wbuff, strlen(wbuff), 1, fpw );
			}
	

		}//end

		fclose( fpw );
		zip_fclose(fp);
		free(buff);

	}//for end

}




void getZipData( struct zip  *handle , int *nfft , int *nfs)
{

	//printf("getZipData start\n");

	char  msg[MSG_LEN];
	



	//get Config
	char *str = "transferFunction/source.xml";
	harkio_XML* src_xml = readZippedXML( handle , str );
	if(src_xml==NULL) 
	{
	    sprintf(msg,"fopen error [%s]", str);
	    harkio_Log_printf(harkio_Log_Error , msg);
	    exit(EXIT_FAILURE);
	}



	*nfft = src_xml->cfg->nfft;
	*nfs  = src_xml->cfg->samplingRate;


	free(src_xml);

	
}	






